#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Collections;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

#region Global Enums

public enum anaPlotAlignVB33 {Left, Right, DoNotPlot}
public enum anaSessionTypeVB33 {ETH, RTH}
public enum anaSessionCountVB33 {First, Second, Third, Hybrid}
public enum anaBandTypeVB33 {Wide, Narrow}

#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// Current Open, High, Low and Fibonacci Levels.
	/// </summary>
	[Description("anaVolatilityBandsV33.")]
	public class anaVolatilityBandsV33 : Indicator
	{
		#region Variables
		private	SolidBrush[]		brushes					= { new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), 
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black), 
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black),  
															new SolidBrush(Color.Black), new SolidBrush(Color.Black), new SolidBrush(Color.Black)};  
		private DateTime 			sessionBegin			= Cbi.Globals.MinDate;
		private DateTime 			sessionEnd 				= Cbi.Globals.MinDate;
		private DateTime			plotStart				= Cbi.Globals.MinDate;
		private DateTime			cacheSessionBeginTmp   	= Cbi.Globals.MinDate;
		private DateTime			cacheSessionEndTmp		= Cbi.Globals.MinDate;
		private DateTime			cacheSessionDate		= Cbi.Globals.MinDate;
		private DateTime			sessionDateTmp			= Cbi.Globals.MinDate;
		private DateTime			currentDate				= Cbi.Globals.MinDate;
		private DateTime			highTime				= Cbi.Globals.MinDate;
		private DateTime			lowTime					= Cbi.Globals.MinDate;
		private DateTime			startTime				= Cbi.Globals.MinDate;
		private double				currentOpen				= 0;
		private	double				currentHigh				= double.MinValue;
		private	double				currentLow				= double.MaxValue;
		private double				currentMidline			= 0;
		private double				fib786					= 0;
		private double				fib618					= 0;
		private double				fib382					= 0;
		private double				fib236					= 0;
		private double				displaySize				= 0;
		private double				averageRange1			= 0;
		private double				averageRange2			= 0;
		private double				adaptedRange1			= 0;
		private double				adaptedRange2			= 0;
		private double				upperTarget1			= 0;
		private double				lowerTarget1			= 0;
		private double				upperTarget2			= 0;
		private double				lowerTarget2			= 0;
		private SolidBrush			textBrush				= new SolidBrush(Color.Red);
		private Font				textFont				= new Font("Arial", 12);
		private string				errorData1				= "Projection Bands can only be displayed on intraday charts.";
		private	string				errorData2				= "Insufficient historical data to calculate Volatility Bands. Please increase chart look back period.";
		private string				errorData3a				= "Insufficient historical data. The Volatility Bands Indicator requires a chart lookback period equivalent to ";
		private string				errorData3b				= " business days.";
		private	string				errorData3c				= "The Volatility Bands Indicator did not find a third intraday session.";
		private float				errorTextWidth			= 0;
        private float				errorTextHeight			= 0;
		private bool				plotOHL					= false;
		private bool				periodOpen				= false;
		private bool				init					= false;
		private bool				skipPublicHoliday		= false; 
		private bool				isPublicHoliday			= false; 
		private bool				rthOHL					= false;
		private bool				showCurrentOpen			= false;
		private bool				showFibonacci			= false;
		private bool				showHiLo				= false;
		private bool				showVolatilityLabels	= false;
		private bool				showVolatilityBands		= true;
		private bool				includeAfterSession		= false;
		private bool				isCurrency				= false;
		private bool				isGlobex				= false;
		private ArrayList			newSessionBarIdxArr1	= new ArrayList();
		private anaPlotAlignVB33	plotLabels				= anaPlotAlignVB33.Right;
		private anaPlotAlignVB33	showRangeData			= anaPlotAlignVB33.Left;
		private anaSessionTypeVB33	currentSession			= anaSessionTypeVB33.ETH;
		private anaSessionCountVB33 selectedSession			= anaSessionCountVB33.Hybrid;
		private anaSessionCountVB33 activeSession			= anaSessionCountVB33.Hybrid;
		private anaBandTypeVB33		bandType				= anaBandTypeVB33.Wide;
		private Data.PivotRange		pivotRangeType1			= PivotRange.Daily;
		private	StringFormat		stringFormatFar			= new StringFormat();
		private	StringFormat		stringFormatNear		= new StringFormat();
		private	StringFormat		stringFormatCenter		= new StringFormat();
		private int					numberOfSessions		= 1;
		private int					sessionCount			= 0;
		private int					rangePeriod				= 0;
		private int					rangePeriod1			= 3;
		private int					rangePeriod2			= 10;
		private int					dayCount				= 0;
		private int					bandWidth				= 100;
		private int					width					= 15;
		private int					opacity					= 6;
		private int					labelPosition			= 10;
		private int					displayFactor			= 1;
		private Color 				rangeColor				= Color.DarkViolet;
		private DateTime			publicHoliday0			= new DateTime (2009,01,19);
		private DateTime			publicHoliday1			= new DateTime (2009,02,16);
		private DateTime			publicHoliday2			= new DateTime (2009,05,25);
		private DateTime			publicHoliday3			= new DateTime (2009,07,03);
		private DateTime			publicHoliday4			= new DateTime (2009,09,07);
		private DateTime			publicHoliday5			= new DateTime (2009,11,26);
		private DateTime			publicHoliday6			= new DateTime (2010,01,18);
		private DateTime			publicHoliday7			= new DateTime (2010,02,15);
		private DateTime			publicHoliday8			= new DateTime (2010,05,31);
		private DateTime			publicHoliday9			= new DateTime (2010,07,05);
		private DateTime			publicHoliday10			= new DateTime (2010,09,06);
		private DateTime			publicHoliday11			= new DateTime (2010,11,25);
		private DateTime			publicHoliday12			= new DateTime (2011,01,17);
		private DateTime			publicHoliday13			= new DateTime (2011,02,21);
		private DateTime			publicHoliday14			= new DateTime (2011,05,30);
		private DateTime			publicHoliday15			= new DateTime (2011,07,04);
		private DateTime			publicHoliday16			= new DateTime (2011,09,05);
		private DateTime			publicHoliday17			= new DateTime (2011,11,24);
		private DateTime[]			publicHoliday			= new DateTime [18];
		private BoolSeries			lowIsLast;
		private double[]			rangeArray				= new double[100];
		private DataSeries			ADR0;
		private DataSeries			ADR1;
		private DataSeries			ADR2;
		private DataSeries			upperRegionUpper;
		private DataSeries			upperRegionLower;
		private DataSeries			lowerRegionUpper;
		private DataSeries			lowerRegionLower;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(new Pen(Color.PeachPuff,1), PlotStyle.Line,"D-Open "));
			Add(new Plot(new Pen(Color.Lime,1), PlotStyle.Line,"D-High "));
			Add(new Plot(new Pen(Color.OrangeRed,1), PlotStyle.Line,"D-Low "));
			Add(new Plot(new Pen(Color.MediumSpringGreen,1), PlotStyle.Line,"D-Mid "));
			Add(new Plot(new Pen(Color.Yellow,1), PlotStyle.Line,"D-78,6 "));
			Add(new Plot(new Pen(Color.Yellow,1), PlotStyle.Line,"D-61,8 "));
			Add(new Plot(new Pen(Color.Yellow,1), PlotStyle.Line,"D-38,2 "));
			Add(new Plot(new Pen(Color.Yellow,1), PlotStyle.Line,"D-23,6 "));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"SAR+"));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"SAR-"));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"LAR+"));
			Add(new Plot(new Pen(Color.DeepSkyBlue,1), PlotStyle.Line,"LAR-"));
			
			Plots[0].Pen.DashStyle = DashStyle.Solid;
			Plots[1].Pen.DashStyle = DashStyle.Solid;
			Plots[2].Pen.DashStyle = DashStyle.Solid;
			Plots[3].Pen.DashStyle = DashStyle.Solid;
			Plots[4].Pen.DashStyle = DashStyle.Solid;
			Plots[5].Pen.DashStyle = DashStyle.Solid;
			Plots[6].Pen.DashStyle = DashStyle.Solid;
			Plots[7].Pen.DashStyle = DashStyle.Solid;
			Plots[8].Pen.DashStyle = DashStyle.Solid;
			Plots[9].Pen.DashStyle = DashStyle.Solid;
			Plots[10].Pen.DashStyle = DashStyle.Solid;
			Plots[11].Pen.DashStyle = DashStyle.Solid;

			for(int i=0; i<18; i++)
				publicHoliday[i] = Cbi.Globals.MinDate;
			for (int i=0; i<100; i++)
				rangeArray[i] = 0.0;
			ADR0 = new DataSeries(this);
			ADR1 = new DataSeries(this);
			ADR2 = new DataSeries(this);
			upperRegionUpper = new DataSeries(this);
			upperRegionLower = new DataSeries(this);
			lowerRegionUpper = new DataSeries(this);
			lowerRegionLower = new DataSeries(this);
			
			AutoScale						= false;
			Overlay							= true;
			ZOrder							= 1;
			MaximumBarsLookBack 			= MaximumBarsLookBack.Infinite;
			stringFormatNear.Alignment 		= StringAlignment.Near;
			stringFormatCenter.Alignment 	= StringAlignment.Center;
			stringFormatFar.Alignment		= StringAlignment.Far;
			
			lowIsLast = new BoolSeries(this);
		}

		/// <summary>
		/// </summary>
		protected override void OnStartUp()
		{
			if (Instrument.MasterInstrument.InstrumentType == InstrumentType.Future &&
				(Instrument.MasterInstrument.Name == "EMD" ||Instrument.MasterInstrument.Name == "ES" || Instrument.MasterInstrument.Name == "NQ" 
				||Instrument.MasterInstrument.Name == "YM"||Instrument.MasterInstrument.Name == "GE" ||Instrument.MasterInstrument.Name == "SR" 
				||Instrument.MasterInstrument.Name == "UB"||Instrument.MasterInstrument.Name == "ZB" ||Instrument.MasterInstrument.Name == "ZF"
				||Instrument.MasterInstrument.Name == "ZN"||Instrument.MasterInstrument.Name == "ZQ" ||Instrument.MasterInstrument.Name == "ZT"
				||Instrument.MasterInstrument.Name == "6A"||Instrument.MasterInstrument.Name == "6B" ||Instrument.MasterInstrument.Name == "6C"
				||Instrument.MasterInstrument.Name == "6E"||Instrument.MasterInstrument.Name == "6J" ||Instrument.MasterInstrument.Name == "6M"
				||Instrument.MasterInstrument.Name == "6N"||Instrument.MasterInstrument.Name == "6S" ||Instrument.MasterInstrument.Name == "E7"
				||Instrument.MasterInstrument.Name == "J7"||Instrument.MasterInstrument.Name == "M6A" ||Instrument.MasterInstrument.Name == "M6B" 
				||Instrument.MasterInstrument.Name == "M6C"||Instrument.MasterInstrument.Name == "M6E" ||Instrument.MasterInstrument.Name == "M6J"
				||Instrument.MasterInstrument.Name == "M6S"||Instrument.MasterInstrument.Name == "CL" ||Instrument.MasterInstrument.Name == "EH"
				||Instrument.MasterInstrument.Name == "GC"||Instrument.MasterInstrument.Name == "HG" ||Instrument.MasterInstrument.Name == "HO"
				||Instrument.MasterInstrument.Name == "NG"||Instrument.MasterInstrument.Name == "QG" ||Instrument.MasterInstrument.Name == "QM"
				||Instrument.MasterInstrument.Name == "RB"||Instrument.MasterInstrument.Name == "SI" ||Instrument.MasterInstrument.Name == "YG" 
				||Instrument.MasterInstrument.Name == "YI"||Instrument.MasterInstrument.Name == "GF" ||Instrument.MasterInstrument.Name == "GPB"
				||Instrument.MasterInstrument.Name == "HE"||Instrument.MasterInstrument.Name == "LE" ||Instrument.MasterInstrument.Name == "YC"
				||Instrument.MasterInstrument.Name == "YK"||Instrument.MasterInstrument.Name == "YW" ||Instrument.MasterInstrument.Name == "ZC"
				||Instrument.MasterInstrument.Name == "ZE"||Instrument.MasterInstrument.Name == "ZL" ||Instrument.MasterInstrument.Name == "ZM"
				||Instrument.MasterInstrument.Name == "ZO"||Instrument.MasterInstrument.Name == "ZR" ||Instrument.MasterInstrument.Name == "ZS"
				||Instrument.MasterInstrument.Name == "ZW"))				
				isGlobex = true;
			if (isGlobex)
			{
				publicHoliday[0] = publicHoliday0;
				publicHoliday[1] = publicHoliday1;
				publicHoliday[2] = publicHoliday2;
				publicHoliday[3] = publicHoliday3;
				publicHoliday[4] = publicHoliday4;
				publicHoliday[5] = publicHoliday5;
				publicHoliday[6] = publicHoliday6;
				publicHoliday[7] = publicHoliday7;
				publicHoliday[8] = publicHoliday8;
				publicHoliday[9] = publicHoliday9;
				publicHoliday[10] = publicHoliday10;
				publicHoliday[11] = publicHoliday11;
				publicHoliday[12] = publicHoliday12;
				publicHoliday[13] = publicHoliday13;
				publicHoliday[14] = publicHoliday14;
				publicHoliday[15] = publicHoliday15;
				publicHoliday[16] = publicHoliday16;
				publicHoliday[17] = publicHoliday17;
			}
			displayFactor = 1;
			if (Instrument.MasterInstrument.InstrumentType == Cbi.InstrumentType.Currency || Instrument.MasterInstrument.Name == "DX"|| Instrument.MasterInstrument.Name == "6A"
				|| Instrument.MasterInstrument.Name == "6B" || Instrument.MasterInstrument.Name == "6C" ||Instrument.MasterInstrument.Name == "6E"
				|| Instrument.MasterInstrument.Name == "6J" || Instrument.MasterInstrument.Name == "6M" || Instrument.MasterInstrument.Name == "6S"
				|| Instrument.MasterInstrument.Name == "6N" || Instrument.MasterInstrument.Name == "E7" || Instrument.MasterInstrument.Name == "J7"
				|| Instrument.MasterInstrument.Name == "M6A" || Instrument.MasterInstrument.Name == "M6B" || Instrument.MasterInstrument.Name == "M6C"
				|| Instrument.MasterInstrument.Name == "M6E" || Instrument.MasterInstrument.Name == "M6J" || Instrument.MasterInstrument.Name == "M6S")
			{
				isCurrency = true;
				if (Instrument.MasterInstrument.Name == "6J")
					displayFactor = 1000000;
				else if (Instrument.MasterInstrument.Name == "USDJPY" || Instrument.MasterInstrument.Name == "EURJPY")
					displayFactor = 1;
				else
					displayFactor = 10000;
			}
			if (selectedSession == anaSessionCountVB33.Hybrid)
			{	
				if(isCurrency)
					activeSession = anaSessionCountVB33.Third;
				else
					activeSession = anaSessionCountVB33.Second;
			}
			else 
				activeSession = selectedSession; 
			if (currentSession == anaSessionTypeVB33.ETH)
			{
				rthOHL = false;
				Plots[8].Name = "FULL+ " + rangePeriod1 + "D";
				Plots[9].Name = "FULL- " + rangePeriod1 + "D";
				Plots[10].Name = "FULL+ " + rangePeriod2 + "D";
				Plots[11].Name = "FULL- " + rangePeriod2 + "D";
			}
			else
			{
				rthOHL = true;
				Plots[8].Name = "RTH+ " + rangePeriod1 + "D";
				Plots[9].Name = "RTH- " + rangePeriod1 + "D";
				Plots[10].Name = "RTH+ " + rangePeriod2 + "D";
				Plots[11].Name = "RTH- " + rangePeriod2 + "D";
			}
			if (Instrument.MasterInstrument.InstrumentType == Cbi.InstrumentType.Currency)
			{	
				if (TickSize == 0.00001 || TickSize == 0.001)
					displaySize = 5* TickSize;
			}
			else
				displaySize = TickSize;
			if (AutoScale)
				AutoScale = false;
			rangePeriod = Math.Max(rangePeriod1, rangePeriod2);
			dayCount = 0;
		}

		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
			if (Bars == null)
				return; 
			if (!Data.BarsType.GetInstance(Bars.Period.Id).IsIntraday)
			{
				DrawTextFixed("errortag1", errorData1, TextPosition.Center, ChartControl.AxisColor, textFont, Color.Transparent,Color.Transparent,0);
				return;
			}
			if (CurrentBar == 0)
			{	
				currentDate = GetLastBarSessionDate(Time[0], Bars, 0, pivotRangeType1);
				Bars.Session.GetNextBeginEnd(Bars, 0, out sessionBegin, out sessionEnd);
				plotStart = sessionEnd;
				if (plotStart.DayOfWeek == DayOfWeek.Saturday)
					plotStart = plotStart.AddDays(2);
				return;
			}
						
			DateTime lastBarTimeStamp1 = GetLastBarSessionDate(Time[0], Bars, 0, pivotRangeType1);
			if (lastBarTimeStamp1 != currentDate)
			{	
				if (FirstTickOfBar)
				{
					if (!skipPublicHoliday)
					{
						dayCount = dayCount +1;
						for (int i = rangePeriod - 1; i>0; i--)
						{
							rangeArray[i] = rangeArray[i-1];
						}
						if (dayCount > 1)
							rangeArray [0] = currentHigh - currentLow;
						if (dayCount > rangePeriod)
						{
							averageRange1 = 0.0;
							for (int i=0; i<rangePeriod1; i++)
								averageRange1 = averageRange1 + rangeArray[i];
							averageRange1 = averageRange1/rangePeriod1;
							adaptedRange1 = bandWidth * averageRange1 / 100;
							averageRange2 = 0.0;
							for (int i=0; i<rangePeriod2; i++)
								averageRange2 = averageRange2 + rangeArray[i];
							averageRange2 = averageRange2/rangePeriod2;
							adaptedRange2 = bandWidth * averageRange2 / 100;
						}
					}
					sessionCount = 1;
					periodOpen = false;
					init = false;
				}
				if (numberOfSessions == 1 || !rthOHL || (rthOHL && activeSession == anaSessionCountVB33.First))
				{
					periodOpen = true;
					init = true;
					if (!skipPublicHoliday || rthOHL)
					{
						currentOpen		= Open[0];
						currentHigh		= High[0];
						currentLow		= Low[0];
						highTime		= Time[0];
						lowTime			= Time[0];
						if (dayCount > rangePeriod && bandType == anaBandTypeVB33.Wide)
						{
							lowerTarget1 = currentHigh - adaptedRange1;
							upperTarget1 = currentLow + adaptedRange1;
							lowerTarget2 = currentHigh - adaptedRange2;
							upperTarget2 = currentLow + adaptedRange2;
							startTime = Time[0];
						}
						else if (dayCount > rangePeriod && bandType == anaBandTypeVB33.Narrow)
						{
							lowerTarget1 = 0.5*(currentHigh + currentLow - adaptedRange1);
							upperTarget1 = 0.5*(currentHigh + currentLow + adaptedRange1);
							lowerTarget2 = 0.5*(currentHigh + currentLow - adaptedRange2);
							upperTarget2 = 0.5*(currentHigh + currentLow + adaptedRange2);
							startTime = Time[0];
						}
					}
					else
					{
						if (High[0] >= currentHigh)
						{
							currentHigh = High[0];
							highTime = Time[0];
							if (dayCount> rangePeriod)
							{
								if (adaptedRange1 > currentHigh - currentLow && bandType == anaBandTypeVB33.Wide)
									lowerTarget1 = currentHigh - adaptedRange1;
								else 
								{
									lowerTarget1 = 0.5*(currentHigh + currentLow - adaptedRange1);
									upperTarget1 = 0.5*(currentHigh + currentLow + adaptedRange1);
								}
								if (adaptedRange2 > currentHigh - currentLow && bandType == anaBandTypeVB33.Wide )
									lowerTarget2 = currentHigh - adaptedRange2;
								else 
								{
									lowerTarget2 = 0.5*(currentHigh + currentLow - adaptedRange2);
									upperTarget2 = 0.5*(currentHigh + currentLow + adaptedRange2);
								}
							}
						}
						if (Low[0] <= currentLow)
						{
							currentLow = Low[0];
							lowTime = Time[0];
							if (dayCount> rangePeriod)
							{
								if (adaptedRange1 > currentHigh - currentLow && bandType == anaBandTypeVB33.Wide)
									upperTarget1 = currentLow + adaptedRange1;
								else 
								{
									lowerTarget1 = 0.5*(currentHigh + currentLow - adaptedRange1);
									upperTarget1 = 0.5*(currentHigh + currentLow + adaptedRange1);
								}
								if (adaptedRange2 > currentHigh - currentLow && bandType == anaBandTypeVB33.Wide)
									upperTarget2 = currentLow + adaptedRange2;
								else 
								{
									lowerTarget2 = 0.5*(currentHigh + currentLow - adaptedRange2);
									upperTarget2 = 0.5*(currentHigh + currentLow + adaptedRange2);
								}
							}
						}
					}
				}
				currentDate = lastBarTimeStamp1;
				skipPublicHoliday = false;
				for (int i =0; i<18; i++)
				{
					if (publicHoliday[i].Date == currentDate)
						skipPublicHoliday = true;
				}
				plotOHL = true;
			}
			else if (Bars.FirstBarOfSession)
			{
				if (FirstTickOfBar)
				{
					sessionCount = sessionCount +1;
					numberOfSessions = Math.Min(3, Math.Max(sessionCount, numberOfSessions));
				}
				if (!rthOHL)
				{
					if (High[0] >= currentHigh)
					{
						currentHigh = High[0];
						highTime = Time[0];
						if (dayCount> rangePeriod)
						{
							if (adaptedRange1 > currentHigh - currentLow && bandType == anaBandTypeVB33.Wide)
								lowerTarget1 = currentHigh - adaptedRange1;
							else 
							{
								lowerTarget1 = 0.5*(currentHigh + currentLow - adaptedRange1);
								upperTarget1 = 0.5*(currentHigh + currentLow + adaptedRange1);
							}
							if (adaptedRange2 > currentHigh - currentLow && bandType == anaBandTypeVB33.Wide)
								lowerTarget2 = currentHigh - adaptedRange2;
							else 
							{
								lowerTarget2 = 0.5*(currentHigh + currentLow - adaptedRange2);
								upperTarget2 = 0.5*(currentHigh + currentLow + adaptedRange2);
							}
						}
					}
					if (Low[0] <= currentLow)
					{
						currentLow = Low[0];
						lowTime = Time[0];
						if (dayCount> rangePeriod)
						{
							if (adaptedRange1 > currentHigh - currentLow && bandType == anaBandTypeVB33.Wide)
								upperTarget1 = currentLow + adaptedRange1;
							else 
							{
								lowerTarget1 = 0.5*(currentHigh + currentLow - adaptedRange1);
								upperTarget1 = 0.5*(currentHigh + currentLow + adaptedRange1);
							}
							if (adaptedRange2 > currentHigh - currentLow && bandType == anaBandTypeVB33.Wide)
								upperTarget2 = currentLow + adaptedRange2;
							else 
							{
								lowerTarget2 = 0.5*(currentHigh + currentLow - adaptedRange2);
								upperTarget2 = 0.5*(currentHigh + currentLow + adaptedRange2);
							}
						}
					}
				}
				else if (rthOHL && ((sessionCount == 2 && activeSession == anaSessionCountVB33.Second)
					|| (sessionCount == 3 && activeSession == anaSessionCountVB33.Third)))
				{
					currentOpen		= Open[0];
					currentHigh		= High[0];
					currentLow		= Low[0];
					highTime		= Time[0];
					lowTime			= Time[0];
					if (dayCount > rangePeriod && bandType == anaBandTypeVB33.Wide)
					{
						lowerTarget1 = High[0] - adaptedRange1;
						upperTarget1 = Low[0] + adaptedRange1;
						lowerTarget2 = High[0] - adaptedRange2;
						upperTarget2 = Low[0] + adaptedRange2;
						startTime = Time[0];
					}
					else if (dayCount > rangePeriod && bandType == anaBandTypeVB33.Narrow)
					{
						lowerTarget1 = 0.5*(currentHigh + currentLow - adaptedRange1);
						upperTarget1 = 0.5*(currentHigh + currentLow + adaptedRange1);
						lowerTarget2 = 0.5*(currentHigh + currentLow - adaptedRange2);
						upperTarget2 = 0.5*(currentHigh + currentLow + adaptedRange2);
						startTime = Time[0];
					}
					periodOpen		= true;
					init 			= true;
				}
				else
					periodOpen = false;
			}
			else if (periodOpen)
			{
				if (High[0] >= currentHigh)
				{
					currentHigh = High[0];
					highTime = Time[0];
					if (dayCount> rangePeriod)
					{
						if (adaptedRange1 > currentHigh - currentLow && bandType == anaBandTypeVB33.Wide)
							lowerTarget1 = currentHigh - adaptedRange1;
						else 
						{
							lowerTarget1 = 0.5*(currentHigh + currentLow - adaptedRange1);
							upperTarget1 = 0.5*(currentHigh + currentLow + adaptedRange1);
						}
						if (adaptedRange2 > currentHigh - currentLow && bandType == anaBandTypeVB33.Wide)
							lowerTarget2 = currentHigh - adaptedRange2;
						else 
						{
							lowerTarget2 = 0.5*(currentHigh + currentLow - adaptedRange2);
							upperTarget2 = 0.5*(currentHigh + currentLow + adaptedRange2);
						}
					}
				}
				if (Low[0] <= currentLow)
				{
					currentLow = Low[0];
					lowTime = Time[0];
					if (dayCount> rangePeriod)
					{
						if (adaptedRange1 > currentHigh - currentLow && bandType == anaBandTypeVB33.Wide)
							upperTarget1 = currentLow + adaptedRange1;
						else 
						{
							lowerTarget1 = 0.5*(currentHigh + currentLow - adaptedRange1);
							upperTarget1 = 0.5*(currentHigh + currentLow + adaptedRange1);
						}
						if (adaptedRange2 > currentHigh - currentLow && bandType == anaBandTypeVB33.Wide)
							upperTarget2 = currentLow + adaptedRange2;
						else 
						{
							lowerTarget2 = 0.5*(currentHigh + currentLow - adaptedRange2);
							upperTarget2 = 0.5*(currentHigh + currentLow + adaptedRange2);
						}
					}
				}
			}

			if (plotOHL && init && !(rthOHL && activeSession == anaSessionCountVB33.Third && numberOfSessions == 2) &&
				!(rthOHL && !includeAfterSession && activeSession == anaSessionCountVB33.Second && sessionCount>2) &&
				!(rthOHL && !includeAfterSession && activeSession == anaSessionCountVB33.First && sessionCount>1))
			{
				if (ShowCurrentOpen)
					CurrentOpen.Set(currentOpen);
				else
					CurrentOpen.Set(0);
				if (ShowHiLo)
				{
					CurrentHigh.Set(currentHigh);
					CurrentLow.Set(currentLow);
					CurrentMidline.Set(Math.Round((currentLow + 0.5*(currentHigh-currentLow))/displaySize)* displaySize);
					if (ShowFibonacci && lowTime >= highTime)				
					{	
						lowIsLast.Set(true);
						Fib786.Set(Math.Round((currentLow + 0.786*(currentHigh-currentLow))/displaySize)*displaySize); 
						Fib618.Set(Math.Round((currentLow + 0.618*(currentHigh-currentLow))/displaySize)*displaySize);
						Fib382.Set(Math.Round((currentLow + 0.382*(currentHigh-currentLow))/displaySize)*displaySize);
						Fib236.Set(Math.Round((currentLow + 0.236*(currentHigh-currentLow))/displaySize)*displaySize);
					}
					else if (ShowFibonacci)
					{
						lowIsLast.Set(false);
						Fib236.Set(Math.Round((currentHigh - 0.236*(currentHigh-currentLow))/displaySize)*displaySize);
						Fib382.Set(Math.Round((currentHigh - 0.382*(currentHigh-currentLow))/displaySize)*displaySize);
						Fib618.Set(Math.Round((currentHigh - 0.618*(currentHigh-currentLow))/displaySize)*displaySize);
						Fib786.Set(Math.Round((currentHigh - 0.786*(currentHigh-currentLow))/displaySize)*displaySize); 
					}
					else
					{
						Fib786.Set(0); 
						Fib618.Set(0); 
						Fib382.Set(0); 
						Fib236.Set(0);
					}
				}
				else
				{
					CurrentHigh.Set(0);
					CurrentLow.Set(0);
					CurrentMidline.Set(0);
					CurrentOpen.Set(0);
					Fib786.Set(0); 
					Fib618.Set(0); 
					Fib382.Set(0); 
					Fib236.Set(0);
				}
				if (dayCount > rangePeriod)
				{
					DrawTextFixed("errortag3", "", TextPosition.Center, textBrush.Color, textFont, Color.Transparent,Color.Transparent,0);
					upperTarget1 = Math.Round(upperTarget1/displaySize)* displaySize;
					lowerTarget1 = Math.Round(lowerTarget1/displaySize)* displaySize;
					upperTarget2 = Math.Round(upperTarget2/displaySize)* displaySize;
					lowerTarget2 = Math.Round(lowerTarget2/displaySize)* displaySize;
					if (upperTarget2 == upperTarget1)
						upperTarget2 = upperTarget2 + displaySize;
					if (lowerTarget2 == lowerTarget1)
						lowerTarget2 = lowerTarget2 - displaySize;
					if (ShowVolatilityLabels)
					{
						SARUp.Set (upperTarget1);
						SARDown.Set (lowerTarget1);
						LARUp.Set (upperTarget2);
						LARDown.Set (lowerTarget2);
					}
					else
					{
						SARUp.Set(0);
						SARDown.Set(0);
						LARUp.Set(0);
						LARDown.Set(0);
					}
					ADR0.Set (Math.Round((currentHigh - currentLow)/displaySize)* displaySize * displayFactor);
					ADR1.Set (Math.Round(averageRange1/displaySize)* displaySize * displayFactor);
					ADR2.Set (Math.Round(averageRange2/displaySize)* displaySize * displayFactor);
					upperRegionUpper.Set(Math.Max(upperTarget1, upperTarget2));
					upperRegionLower.Set(Math.Min(upperTarget1, upperTarget2));
					lowerRegionUpper.Set(Math.Max(lowerTarget1, lowerTarget2));
					lowerRegionLower.Set(Math.Min(lowerTarget1, lowerTarget2));
					if (ShowVolatilityBands)
					{
						DrawRegion("upper" + dayCount, startTime, Time[0],upperRegionUpper, upperRegionLower,Color.Transparent, RangeColor, Opacity); 
						DrawRegion("lower" + dayCount, startTime, Time[0],lowerRegionUpper, lowerRegionLower,Color.Transparent, RangeColor, Opacity);
					}
				}
				else
				{
					string errorData3 = errorData3a + (rangePeriod+1) + errorData3b;
					DrawTextFixed("errortag3", errorData3, TextPosition.Center, ChartControl.AxisColor, textFont, Color.Transparent,Color.Transparent,0);
					ADR0.Set(0);
					ADR1.Set(0);
					ADR2.Set(0);
					SARUp.Set(0);
					SARDown.Set(0);
					LARUp.Set(0);
					LARDown.Set(0);
				}
			}
			else
			{
				CurrentHigh.Set(0);
				CurrentLow.Set(0);
				CurrentMidline.Set(0);
				CurrentOpen.Set(0);
				Fib786.Set(0); 
				Fib618.Set(0); 
				Fib382.Set(0); 
				Fib236.Set(0);
				ADR0.Set(0);
				if (dayCount > rangePeriod && !(rthOHL && activeSession == anaSessionCountVB33.Third && numberOfSessions == 2))
				{
					ADR1.Set (Math.Round(averageRange1/displaySize)* displaySize * displayFactor);
					ADR2.Set (Math.Round(averageRange2/displaySize)* displaySize * displayFactor);
				}
				else
				{
					if (rthOHL && activeSession == anaSessionCountVB33.Third && numberOfSessions == 2)
						DrawTextFixed("errortag3", errorData3c, TextPosition.Center, ChartControl.AxisColor, textFont, Color.Transparent,Color.Transparent,0);
					ADR1.Set(0);
					ADR2.Set(0);
				}
				SARUp.Set(0);
				SARDown.Set(0);
				LARUp.Set(0);
				LARDown.Set(0);
			}
		}


		#region Properties
		/// <summary>
		/// </summary>
		[Description("Session used for calculating OHL")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Session")]
		public anaSessionTypeVB33 CurrentSession
		{
			get { return currentSession; }
			set { currentSession = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Session used for calculating OHL")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Session # for RTH")]
		public anaSessionCountVB33 SelectedSession
		{
			get { return selectedSession; }
			set { selectedSession = value; }
		}

		/// <summary>
		/// </summary>
		[Description("Option to show CurrentOpen")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("Show Current Open")]
		public bool ShowCurrentOpen 
		{
			get { return showCurrentOpen; }
			set { showCurrentOpen = value; }
		}
		
		[Description("Option to show Fibonacci Lines")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("Show Dynamic Fibs")]
		public bool ShowFibonacci 
		{
			get { return showFibonacci; }
			set { showFibonacci = value; }
		}
		
		[Description("Option to show current day's High and Low")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("Show High/Low")]
		public bool ShowHiLo 
		{
			get { return showHiLo; }
			set { showHiLo = value; }
		}
		
		[Description("Option to show volatility bands")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("Show Projection Bands")]
		public bool ShowVolatilityBands
		{
			get { return showVolatilityBands; }
			set { showVolatilityBands = value; }
		}
		
		[Description("Option to show volatility band labels")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("Show Projection Labels")]
		public bool ShowVolatilityLabels 
		{
			get { return showVolatilityLabels; }
			set { showVolatilityLabels = value; }
		}
		
		[Description("Option to show current day's range and average ranges")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("Show Range Data")]
		public anaPlotAlignVB33 ShowRangeData 
		{
			get { return showRangeData; }
			set { showRangeData = value; }
		}
		
		[Description("Option to display indicators in RTH mode for the following sessions")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("Include After Session")]
		public bool IncludeAfterSession
		{
			get { return includeAfterSession; }
			set { includeAfterSession = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Option where to plot labels")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Label Position")]
		public anaPlotAlignVB33 PlotLabels
		{
			get { return plotLabels; }
			set { plotLabels = value; }
		}
			
		/// <summary>
		/// </summary>
		[Description("Label distance from line.")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Label Offset")]
		public int LabelPosition
		{
			get { return labelPosition; }
			set { labelPosition = Math.Max(1, value); }
		}
	
		/// <summary>
		/// </summary>
		[Description("# of days used to calculate average daily range.")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Period(1) for Daily Range")]
		public int RangePeriod1
		{
			get { return rangePeriod1; }
			set { rangePeriod1 = Math.Min(100,Math.Max(1, value)); }
		}

		/// <summary>
		/// </summary>
		[Description("# of days used to calculate average daily range.")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Period(2) for Daily Range")]
		public int RangePeriod2
		{
			get { return rangePeriod2; }
			set { rangePeriod2 = Math.Min(100,Math.Max(1, value)); }
		}

		/// <summary>
		/// </summary>
		[Description("Percent of average daily range used to calculate the bands")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Band Range in % of ADR")]
		public int BandWidth
		{
			get { return bandWidth; }
			set { bandWidth = Math.Min(400,Math.Max(10, value)); }
		}

		/// <summary>
		/// </summary>
		[Description("Option to plot wide or narrow bands")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Band Type")]
		public anaBandTypeVB33 BandType
		{
			get { return bandType; }
			set { bandType = value; }
		}
			
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries CurrentOpen
		{
			get { return Values[0]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries CurrentHigh
		{
			get { return Values[1]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries CurrentLow
		{
			get { return Values[2]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries CurrentMidline
		{
			get { return Values[3]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries Fib786
		{
			get { return Values[4]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries Fib618
		{
			get { return Values[5]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries Fib382
		{
			get { return Values[6]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries Fib236
		{
			get { return Values[7]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries SARUp
		{
			get { return Values[8]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries SARDown
		{
			get { return Values[9]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries LARUp
		{
			get { return Values[10]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries LARDown
		{
			get { return Values[11]; }
		}

		/// <summary>
		/// </summary>
		[Description("Width of the OHL lines as # of bars.")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("Line Width")]
		public int Width
		{
			get { return width; }
			set { width = Math.Max(0, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("Opacity of Ranges ")]
		[Category("Parameters")]
		[Gui.Design.DisplayNameAttribute("Range Opacity")]
		public int Opacity
		{
			get { return opacity; }
			set { opacity = Math.Min(10, Math.Max(1, value)); }
		}

		[Description("Select Expansion Range Color")]
		[Category("Colors")]
		[Gui.Design.DisplayName("Expansion Range Color")]
		public Color RangeColor
		{
			get { return rangeColor; }
			set { rangeColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string RangeColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(rangeColor); }
			set { rangeColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
        [Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 01 /No Trade Date")]
		public DateTime PublicHoliday0
		{
			get { return publicHoliday0;}
			set { publicHoliday0 = value;}
		}	
	
		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
        [Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 02 /No Trade Date")]
		public DateTime PublicHoliday1
		{
			get { return publicHoliday1;}
			set { publicHoliday1 = value;}
		}	
		
		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 03 /No Trade Date")]
		public DateTime PublicHoliday2
		{
			get { return publicHoliday2;}
			set { publicHoliday2 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 04 /No Trade Date")]
		public DateTime PublicHoliday3
		{
			get { return publicHoliday3;}
			set { publicHoliday3 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 05 /No Trade Date")]
		public DateTime PublicHoliday4
		{
			get { return publicHoliday4;}
			set { publicHoliday4 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 06 /No Trade Date")]
		public DateTime PublicHoliday5
		{
			get { return publicHoliday5;}
			set { publicHoliday5 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 07 /No Trade Date")]
		public DateTime PublicHoliday6
		{
			get { return publicHoliday6;}
			set { publicHoliday6 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 08 /No Trade Date")]
		public DateTime PublicHoliday7
		{
			get { return publicHoliday7;}
			set { publicHoliday7 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 09 /No Trade Date")]
		public DateTime PublicHoliday8
		{
			get { return publicHoliday8;}
			set { publicHoliday8 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 10 /No Trade Date")]
		public DateTime PublicHoliday9
		{
			get { return publicHoliday9;}
			set { publicHoliday9 = value;}
		}	
		
		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 11 /No Trade Date")]
		public DateTime PublicHoliday10
		{
			get { return publicHoliday10;}
			set { publicHoliday10 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 12 /No Trade Date")]
		public DateTime PublicHoliday11
		{
			get { return publicHoliday11;}
			set { publicHoliday11 = value;}
		}	
		
		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 13 /No Trade Date")]
		public DateTime PublicHoliday12
		{
			get { return publicHoliday12;}
			set { publicHoliday12 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 14 /No Trade Date")]
		public DateTime PublicHoliday13
		{
			get { return publicHoliday13;}
			set { publicHoliday13 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 15 /No Trade Date")]
		public DateTime PublicHoliday14
		{
			get { return publicHoliday14;}
			set { publicHoliday14 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 16 /No Trade Date")]
		public DateTime PublicHoliday15
		{
			get { return publicHoliday15;}
			set { publicHoliday15 = value;}
		}	
		
		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 17 /No Trade Date")]
		public DateTime PublicHoliday16
		{
			get { return publicHoliday16;}
			set { publicHoliday16 = value;}
		}	

		///<summary
		///</summary>
		[Description("Enter dates for public holidays with no session close.")]
		[Category("Globex Open with Settlement Next Day")]
		[Gui.Design.DisplayNameAttribute("Holiday 18 /No Trade Date")]
		public DateTime PublicHoliday17
		{
			get { return publicHoliday17;}
			set { publicHoliday17 = value;}
		}	
		#endregion
		
		#region Miscellaneous

		public override string FormatPriceMarker(double price)
		{
			double trunc = Math.Truncate(price);
			int fraction = Convert.ToInt32(320 * Math.Abs(price - trunc) - 0.0001); // rounding down for ZF and ZT
			string priceMarker = "";
			if (TickSize == 0.03125) 
			{
				fraction = fraction/10;
				if (fraction < 10)
					priceMarker = trunc.ToString() + "'0" + fraction.ToString();
				else 
					priceMarker = trunc.ToString() + "'" + fraction.ToString();
			}
			else if (TickSize == 0.015625 || TickSize == 0.0078125)
			{
				if (fraction < 10)
					priceMarker = trunc.ToString() + "'00" + fraction.ToString();
				else if (fraction < 100)
					priceMarker = trunc.ToString() + "'0" + fraction.ToString();
				else	
					priceMarker = trunc.ToString() + "'" + fraction.ToString();
			}
			else
				priceMarker = price.ToString(Gui.Globals.GetTickFormatString(TickSize));
			return priceMarker;
		}		
		
		private DateTime GetLastBarSessionDate(DateTime time, Data.Bars bars, int barsAgo, PivotRange pivotRange)
		{
			if (time > cacheSessionEndTmp)
			{
				isPublicHoliday = false;
				for (int i =0; i<18; i++)
				{
					if (publicHoliday[i].Date == sessionDateTmp)
						isPublicHoliday = true;
				}
				Bars.Session.GetNextBeginEnd(bars, barsAgo, out cacheSessionBeginTmp, out cacheSessionEndTmp);
				if (Bars.BarsType.IsIntraday)
					sessionDateTmp = Bars.GetTradingDayFromLocal(time);
				else
					sessionDateTmp = time.Date;
				
				if(cacheSessionDate != sessionDateTmp || (rthOHL && sessionCount == 1 && activeSession == anaSessionCountVB33.Second)
					|| (rthOHL && sessionCount ==2 && activeSession == anaSessionCountVB33.Third)) 
					{
						cacheSessionDate = sessionDateTmp;
						if ((!isPublicHoliday || rthOHL) && (newSessionBarIdxArr1.Count == 0 
							|| (newSessionBarIdxArr1.Count > 0 && CurrentBar > (int) newSessionBarIdxArr1[newSessionBarIdxArr1.Count - 1])))
								newSessionBarIdxArr1.Add(CurrentBar);
					}
			}
			return sessionDateTmp;
		}
		
		
		internal void InvalidateNow()
		{
			if (Disposed || ChartControl == null)
				return;

			ChartControl.Invalidate(true);
		}

		/// <summary>
        /// Overload this method to handle the termination of an indicator. Use this method to dispose of any resources vs overloading the Dispose() method.
		/// </summary>
		protected override void OnTermination()
		{
			textBrush.Dispose();
			foreach (SolidBrush solidBrush in brushes)
				solidBrush.Dispose();
			stringFormatCenter.Dispose();
			stringFormatFar.Dispose();	
			stringFormatNear.Dispose();
		}
	
		/// <summary>
		/// </summary>
		/// <param name="graphics"></param>
		/// <param name="bounds"></param>
		/// <param name="min"></param>
		/// <param name="max"></param>
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			if (Bars == null || ChartControl == null)
				return;

			// plot error if data not complete
			DateTime lastBarSessionBegin = Cbi.Globals.MinDate;
			DateTime lastBarSessionEnd = Cbi.Globals.MinDate;
			if (ChartControl.EquidistantBars)
				Bars.Session.GetNextBeginEnd(Bars.Get(Math.Min(Bars.Count - 1, ChartControl.LastBarPainted)).Time, out lastBarSessionBegin, out lastBarSessionEnd);
			else
				Bars.Session.GetNextBeginEnd(Bars.Get(Math.Min(Bars.Count - 1, Bars.GetBar(ChartControl.LastBarTimePainted))).Time, out lastBarSessionBegin, out lastBarSessionEnd);
			textBrush.Color = ChartControl.AxisColor;
			SizeF errorSize	= graphics.MeasureString(errorData2, textFont);
			errorTextWidth	= errorSize.Width + 5;
			errorTextHeight	= errorSize.Height + 5;
			if (Bars.Count > 0 && plotStart.Date >= lastBarSessionEnd.Date)
				graphics.DrawString(errorData2, textFont, textBrush, bounds.X + bounds.Width - errorTextWidth, bounds.Y + bounds.Height - errorTextHeight, stringFormatNear);
				
			bool retraceFromLow = lowIsLast.Get(this.LastBarIndexPainted);
			if (ShowFibonacci)
			{
				if (retraceFromLow)
				{
					Plots[4].Name = "D-78.6";
					Plots[5].Name = "D-61.8";
					Plots[6].Name = "D-38.2";
					Plots[7].Name = "D-23.6";
				}
				else
				{
					Plots[4].Name = "D-23.6";
					Plots[5].Name = "D-38.2";
					Plots[6].Name = "D-61.8";
					Plots[7].Name = "D-78.6";
				}
			}
			
			double averageDR0 = ADR0.Get(this.LastBarIndexPainted);
			double averageDR1 = ADR1.Get(this.LastBarIndexPainted);
			double averageDR2 = ADR2.Get(this.LastBarIndexPainted);
			string rangeData0 = "Current Day Range = ";
			string rangeData1 = "ADR [" + rangePeriod1 + "] = ";
			string rangeData2 = "ADR [" + rangePeriod2 + "] = ";
			if (TickSize == 0.03125) 
			{
				double trunc0 = Math.Truncate(averageDR0); 
				double trunc1 = Math.Truncate(averageDR1); 
				double trunc2 = Math.Truncate(averageDR2);
				int fraction0 = Convert.ToInt32(32 * Math.Abs(averageDR0 - trunc0) -0.0001);
				int fraction1 = Convert.ToInt32(32 * Math.Abs(averageDR1 - trunc1) -0.0001);
				int fraction2 = Convert.ToInt32(32 * Math.Abs(averageDR2 - trunc2) -0.0001);
				if (fraction0 < 10)
					rangeData0 = rangeData0 + trunc0.ToString() + "'0" + fraction0.ToString();
				else 
					rangeData0 = rangeData0 + trunc0.ToString() + "'" + fraction0.ToString();
				if (fraction1 < 10)
					rangeData1 = rangeData1 + trunc1.ToString() + "'0" + fraction1.ToString();
				else 
					rangeData1 = rangeData1 + trunc1.ToString() + "'" + fraction1.ToString();
				if (fraction2 < 10)
					rangeData2 = rangeData2 + trunc2.ToString() + "'0" + fraction2.ToString();
				else 
					rangeData2 = rangeData2 + trunc2.ToString() + "'" + fraction2.ToString();
			}
			else if (TickSize == 0.015625 || TickSize == 0.0078125)
			{
				double trunc0 = Math.Truncate(averageDR0); 
				double trunc1 = Math.Truncate(averageDR1); 
				double trunc2 = Math.Truncate(averageDR2);
				int fraction0 = Convert.ToInt32(320 * Math.Abs(averageDR0 - trunc0) -0.0001);
				int fraction1 = Convert.ToInt32(320 * Math.Abs(averageDR1 - trunc1) -0.0001);
				int fraction2 = Convert.ToInt32(320 * Math.Abs(averageDR2 - trunc2) -0.0001);
				if (fraction0 < 10)
					rangeData0 = rangeData0 + trunc0.ToString() + "'00" + fraction0.ToString();
				else if (fraction0 < 100)
					rangeData0 = rangeData0 + trunc0.ToString() + "'0" + fraction0.ToString();
				else 
					rangeData0 = rangeData0 + trunc0.ToString() + "'" + fraction0.ToString();
				if (fraction1 < 10)
					rangeData1 = rangeData1 + trunc1.ToString() + "'00" + fraction1.ToString();
				else if (fraction1 < 100)
					rangeData1 = rangeData1 + trunc1.ToString() + "'0" + fraction1.ToString();
				else 
					rangeData1 = rangeData1 + trunc1.ToString() + "'" + fraction1.ToString();
				if (fraction2 < 10)
					rangeData2 = rangeData2 + trunc2.ToString() + "'00" + fraction2.ToString();
				else if (fraction2 < 100)
					rangeData2 = rangeData2 + trunc2.ToString() + "'0" + fraction2.ToString();
				else 
					rangeData2 = rangeData2 + trunc2.ToString() + "'" + fraction2.ToString();
			}
			else if (isCurrency)
			{
				rangeData0 = rangeData0 + averageDR0.ToString();	
				rangeData1 = rangeData1 + averageDR1.ToString();	
				rangeData2 = rangeData2 + averageDR2.ToString();	
			}	
			else	
			{
				rangeData0 = rangeData0 + averageDR0.ToString(Gui.Globals.GetTickFormatString(TickSize));	
				rangeData1 = rangeData1 + averageDR1.ToString(Gui.Globals.GetTickFormatString(TickSize));	
				rangeData2 = rangeData2 + averageDR2.ToString(Gui.Globals.GetTickFormatString(TickSize));	
			}
			if (showRangeData == anaPlotAlignVB33.Left)
			{
				graphics.DrawString(rangeData0, textFont, textBrush, bounds.X + 25, bounds.Y + textFont.Height + 20, stringFormatNear);
				graphics.DrawString(rangeData1, textFont, textBrush, bounds.X + 25, bounds.Y + 2*(textFont.Height) + 25, stringFormatNear);
				graphics.DrawString(rangeData2, textFont, textBrush, bounds.X + 25, bounds.Y + 3*(textFont.Height) + 30, stringFormatNear);
			}
			else if (showRangeData == anaPlotAlignVB33.Right)
			{
				SizeF rangeSize	= graphics.MeasureString(rangeData0, textFont);
				graphics.DrawString(rangeData0, textFont, textBrush, bounds.X + bounds.Width - rangeSize.Width - 25, bounds.Y + textFont.Height + 20, stringFormatNear);
				graphics.DrawString(rangeData1, textFont, textBrush, bounds.X + bounds.Width - rangeSize.Width - 25, bounds.Y + 2*(textFont.Height) + 25, stringFormatNear);
				graphics.DrawString(rangeData2, textFont, textBrush, bounds.X + bounds.Width - rangeSize.Width - 25, bounds.Y + 3*(textFont.Height) + 30, stringFormatNear);
			}
			
			int	barWidth  	= ChartControl.ChartStyle.GetBarPaintWidth(Bars.BarsData.ChartStyle.BarWidthUI);
			for (int seriesCount = 0; seriesCount < Values.Length; seriesCount++)
			{
				SolidBrush		brush				= brushes[seriesCount];
				int				firstBarIdxToPaint	= -1;
				int				firstX				= -1;
				int				lastX				= -1;
				int				lastY				= -1;
				SmoothingMode	oldSmoothingMode	= graphics.SmoothingMode;
				Gui.Chart.Plot	plot				= Plots[seriesCount];
				DataSeries		series				= (DataSeries) Values[seriesCount];

				for (int i = newSessionBarIdxArr1.Count - 1; i >= 0; i--)
				{
					int prevSessionBreakIdx = (int) newSessionBarIdxArr1[i];
					if (prevSessionBreakIdx <= this.LastBarIndexPainted)
					{
						firstBarIdxToPaint = prevSessionBreakIdx;
						break;
					}
				}

				using (GraphicsPath	path = new GraphicsPath()) 
				{
					if (brush.Color != plot.Pen.Color)	
						brush = new SolidBrush(plot.Pen.Color);

					for (int idx = this.LastBarIndexPainted; idx >= Math.Max(this.FirstBarIndexPainted, this.LastBarIndexPainted - Width); idx--)
					{
						if (idx - Displacement < 0 || idx - Displacement >= Bars.Count || (!ChartControl.ShowBarsRequired && idx - Displacement < BarsRequired))
							continue;
						else if (!series.IsValidPlot(idx))
							continue;

						if (idx < firstBarIdxToPaint)
							break;

						double	val = series.Get(idx);
						int		x	= ChartControl.GetXByBarIdx(BarsArray[0], idx);
						int		y	= ChartControl.GetYByValue(this, val);

						if (lastX >= 0)
						{
							if (y != lastY) // Problem here is, that last bar of old day has date of new day
								y = lastY;
							path.AddLine(lastX - plot.Pen.Width / 2, lastY, x - plot.Pen.Width / 2, y);
						}
						lastX	= x;
						lastY	= y;
						if (idx == this.LastBarIndexPainted || idx == this.LastBarIndexPainted-1)
							firstX	= x;
					}
					graphics.SmoothingMode = SmoothingMode.AntiAlias;
					graphics.DrawPath(plot.Pen, path);
					graphics.SmoothingMode = oldSmoothingMode;
					if (PlotLabels == anaPlotAlignVB33.Right)
					graphics.DrawString(plot.Name, ChartControl.Font, brush, firstX + LabelPosition + 30, lastY - ChartControl.Font.GetHeight() / 2, stringFormatNear);
					if (PlotLabels == anaPlotAlignVB33.Left)
					graphics.DrawString(plot.Name, ChartControl.Font, brush, lastX - LabelPosition, lastY - ChartControl.Font.GetHeight() / 2, stringFormatFar);
				}
			}
		}

		private DateTime RoundUpTimeToPeriodTime(DateTime time, PivotRange pivotRange)
		{
			if (pivotRange == PivotRange.Weekly)
			{
				DateTime periodStart = time.AddDays((6 - (((int) time.DayOfWeek) + 1) % 7));
				return periodStart.Date.AddDays(System.Math.Ceiling(System.Math.Ceiling(time.Date.Subtract(periodStart.Date).TotalDays) / 7) * 7).Date;
			}
			else if (pivotRange == PivotRange.Monthly)
			{
				DateTime result = new DateTime(time.Year, time.Month, 1); 
				return result.AddMonths(1).AddDays(-1);
			}
			else
				return time;
		}
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private anaVolatilityBandsV33[] cacheanaVolatilityBandsV33 = null;

        private static anaVolatilityBandsV33 checkanaVolatilityBandsV33 = new anaVolatilityBandsV33();

        /// <summary>
        /// anaVolatilityBandsV33.
        /// </summary>
        /// <returns></returns>
        public anaVolatilityBandsV33 anaVolatilityBandsV33(anaBandTypeVB33 bandType, int bandWidth, anaSessionTypeVB33 currentSession, bool includeAfterSession, int labelPosition, int opacity, anaPlotAlignVB33 plotLabels, int rangePeriod1, int rangePeriod2, anaSessionCountVB33 selectedSession, bool showCurrentOpen, bool showFibonacci, bool showHiLo, anaPlotAlignVB33 showRangeData, bool showVolatilityBands, bool showVolatilityLabels, int width)
        {
            return anaVolatilityBandsV33(Input, bandType, bandWidth, currentSession, includeAfterSession, labelPosition, opacity, plotLabels, rangePeriod1, rangePeriod2, selectedSession, showCurrentOpen, showFibonacci, showHiLo, showRangeData, showVolatilityBands, showVolatilityLabels, width);
        }

        /// <summary>
        /// anaVolatilityBandsV33.
        /// </summary>
        /// <returns></returns>
        public anaVolatilityBandsV33 anaVolatilityBandsV33(Data.IDataSeries input, anaBandTypeVB33 bandType, int bandWidth, anaSessionTypeVB33 currentSession, bool includeAfterSession, int labelPosition, int opacity, anaPlotAlignVB33 plotLabels, int rangePeriod1, int rangePeriod2, anaSessionCountVB33 selectedSession, bool showCurrentOpen, bool showFibonacci, bool showHiLo, anaPlotAlignVB33 showRangeData, bool showVolatilityBands, bool showVolatilityLabels, int width)
        {
            if (cacheanaVolatilityBandsV33 != null)
                for (int idx = 0; idx < cacheanaVolatilityBandsV33.Length; idx++)
                    if (cacheanaVolatilityBandsV33[idx].BandType == bandType && cacheanaVolatilityBandsV33[idx].BandWidth == bandWidth && cacheanaVolatilityBandsV33[idx].CurrentSession == currentSession && cacheanaVolatilityBandsV33[idx].IncludeAfterSession == includeAfterSession && cacheanaVolatilityBandsV33[idx].LabelPosition == labelPosition && cacheanaVolatilityBandsV33[idx].Opacity == opacity && cacheanaVolatilityBandsV33[idx].PlotLabels == plotLabels && cacheanaVolatilityBandsV33[idx].RangePeriod1 == rangePeriod1 && cacheanaVolatilityBandsV33[idx].RangePeriod2 == rangePeriod2 && cacheanaVolatilityBandsV33[idx].SelectedSession == selectedSession && cacheanaVolatilityBandsV33[idx].ShowCurrentOpen == showCurrentOpen && cacheanaVolatilityBandsV33[idx].ShowFibonacci == showFibonacci && cacheanaVolatilityBandsV33[idx].ShowHiLo == showHiLo && cacheanaVolatilityBandsV33[idx].ShowRangeData == showRangeData && cacheanaVolatilityBandsV33[idx].ShowVolatilityBands == showVolatilityBands && cacheanaVolatilityBandsV33[idx].ShowVolatilityLabels == showVolatilityLabels && cacheanaVolatilityBandsV33[idx].Width == width && cacheanaVolatilityBandsV33[idx].EqualsInput(input))
                        return cacheanaVolatilityBandsV33[idx];

            lock (checkanaVolatilityBandsV33)
            {
                checkanaVolatilityBandsV33.BandType = bandType;
                bandType = checkanaVolatilityBandsV33.BandType;
                checkanaVolatilityBandsV33.BandWidth = bandWidth;
                bandWidth = checkanaVolatilityBandsV33.BandWidth;
                checkanaVolatilityBandsV33.CurrentSession = currentSession;
                currentSession = checkanaVolatilityBandsV33.CurrentSession;
                checkanaVolatilityBandsV33.IncludeAfterSession = includeAfterSession;
                includeAfterSession = checkanaVolatilityBandsV33.IncludeAfterSession;
                checkanaVolatilityBandsV33.LabelPosition = labelPosition;
                labelPosition = checkanaVolatilityBandsV33.LabelPosition;
                checkanaVolatilityBandsV33.Opacity = opacity;
                opacity = checkanaVolatilityBandsV33.Opacity;
                checkanaVolatilityBandsV33.PlotLabels = plotLabels;
                plotLabels = checkanaVolatilityBandsV33.PlotLabels;
                checkanaVolatilityBandsV33.RangePeriod1 = rangePeriod1;
                rangePeriod1 = checkanaVolatilityBandsV33.RangePeriod1;
                checkanaVolatilityBandsV33.RangePeriod2 = rangePeriod2;
                rangePeriod2 = checkanaVolatilityBandsV33.RangePeriod2;
                checkanaVolatilityBandsV33.SelectedSession = selectedSession;
                selectedSession = checkanaVolatilityBandsV33.SelectedSession;
                checkanaVolatilityBandsV33.ShowCurrentOpen = showCurrentOpen;
                showCurrentOpen = checkanaVolatilityBandsV33.ShowCurrentOpen;
                checkanaVolatilityBandsV33.ShowFibonacci = showFibonacci;
                showFibonacci = checkanaVolatilityBandsV33.ShowFibonacci;
                checkanaVolatilityBandsV33.ShowHiLo = showHiLo;
                showHiLo = checkanaVolatilityBandsV33.ShowHiLo;
                checkanaVolatilityBandsV33.ShowRangeData = showRangeData;
                showRangeData = checkanaVolatilityBandsV33.ShowRangeData;
                checkanaVolatilityBandsV33.ShowVolatilityBands = showVolatilityBands;
                showVolatilityBands = checkanaVolatilityBandsV33.ShowVolatilityBands;
                checkanaVolatilityBandsV33.ShowVolatilityLabels = showVolatilityLabels;
                showVolatilityLabels = checkanaVolatilityBandsV33.ShowVolatilityLabels;
                checkanaVolatilityBandsV33.Width = width;
                width = checkanaVolatilityBandsV33.Width;

                if (cacheanaVolatilityBandsV33 != null)
                    for (int idx = 0; idx < cacheanaVolatilityBandsV33.Length; idx++)
                        if (cacheanaVolatilityBandsV33[idx].BandType == bandType && cacheanaVolatilityBandsV33[idx].BandWidth == bandWidth && cacheanaVolatilityBandsV33[idx].CurrentSession == currentSession && cacheanaVolatilityBandsV33[idx].IncludeAfterSession == includeAfterSession && cacheanaVolatilityBandsV33[idx].LabelPosition == labelPosition && cacheanaVolatilityBandsV33[idx].Opacity == opacity && cacheanaVolatilityBandsV33[idx].PlotLabels == plotLabels && cacheanaVolatilityBandsV33[idx].RangePeriod1 == rangePeriod1 && cacheanaVolatilityBandsV33[idx].RangePeriod2 == rangePeriod2 && cacheanaVolatilityBandsV33[idx].SelectedSession == selectedSession && cacheanaVolatilityBandsV33[idx].ShowCurrentOpen == showCurrentOpen && cacheanaVolatilityBandsV33[idx].ShowFibonacci == showFibonacci && cacheanaVolatilityBandsV33[idx].ShowHiLo == showHiLo && cacheanaVolatilityBandsV33[idx].ShowRangeData == showRangeData && cacheanaVolatilityBandsV33[idx].ShowVolatilityBands == showVolatilityBands && cacheanaVolatilityBandsV33[idx].ShowVolatilityLabels == showVolatilityLabels && cacheanaVolatilityBandsV33[idx].Width == width && cacheanaVolatilityBandsV33[idx].EqualsInput(input))
                            return cacheanaVolatilityBandsV33[idx];

                anaVolatilityBandsV33 indicator = new anaVolatilityBandsV33();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.BandType = bandType;
                indicator.BandWidth = bandWidth;
                indicator.CurrentSession = currentSession;
                indicator.IncludeAfterSession = includeAfterSession;
                indicator.LabelPosition = labelPosition;
                indicator.Opacity = opacity;
                indicator.PlotLabels = plotLabels;
                indicator.RangePeriod1 = rangePeriod1;
                indicator.RangePeriod2 = rangePeriod2;
                indicator.SelectedSession = selectedSession;
                indicator.ShowCurrentOpen = showCurrentOpen;
                indicator.ShowFibonacci = showFibonacci;
                indicator.ShowHiLo = showHiLo;
                indicator.ShowRangeData = showRangeData;
                indicator.ShowVolatilityBands = showVolatilityBands;
                indicator.ShowVolatilityLabels = showVolatilityLabels;
                indicator.Width = width;
                Indicators.Add(indicator);
                indicator.SetUp();

                anaVolatilityBandsV33[] tmp = new anaVolatilityBandsV33[cacheanaVolatilityBandsV33 == null ? 1 : cacheanaVolatilityBandsV33.Length + 1];
                if (cacheanaVolatilityBandsV33 != null)
                    cacheanaVolatilityBandsV33.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheanaVolatilityBandsV33 = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// anaVolatilityBandsV33.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.anaVolatilityBandsV33 anaVolatilityBandsV33(anaBandTypeVB33 bandType, int bandWidth, anaSessionTypeVB33 currentSession, bool includeAfterSession, int labelPosition, int opacity, anaPlotAlignVB33 plotLabels, int rangePeriod1, int rangePeriod2, anaSessionCountVB33 selectedSession, bool showCurrentOpen, bool showFibonacci, bool showHiLo, anaPlotAlignVB33 showRangeData, bool showVolatilityBands, bool showVolatilityLabels, int width)
        {
            return _indicator.anaVolatilityBandsV33(Input, bandType, bandWidth, currentSession, includeAfterSession, labelPosition, opacity, plotLabels, rangePeriod1, rangePeriod2, selectedSession, showCurrentOpen, showFibonacci, showHiLo, showRangeData, showVolatilityBands, showVolatilityLabels, width);
        }

        /// <summary>
        /// anaVolatilityBandsV33.
        /// </summary>
        /// <returns></returns>
        public Indicator.anaVolatilityBandsV33 anaVolatilityBandsV33(Data.IDataSeries input, anaBandTypeVB33 bandType, int bandWidth, anaSessionTypeVB33 currentSession, bool includeAfterSession, int labelPosition, int opacity, anaPlotAlignVB33 plotLabels, int rangePeriod1, int rangePeriod2, anaSessionCountVB33 selectedSession, bool showCurrentOpen, bool showFibonacci, bool showHiLo, anaPlotAlignVB33 showRangeData, bool showVolatilityBands, bool showVolatilityLabels, int width)
        {
            return _indicator.anaVolatilityBandsV33(input, bandType, bandWidth, currentSession, includeAfterSession, labelPosition, opacity, plotLabels, rangePeriod1, rangePeriod2, selectedSession, showCurrentOpen, showFibonacci, showHiLo, showRangeData, showVolatilityBands, showVolatilityLabels, width);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// anaVolatilityBandsV33.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.anaVolatilityBandsV33 anaVolatilityBandsV33(anaBandTypeVB33 bandType, int bandWidth, anaSessionTypeVB33 currentSession, bool includeAfterSession, int labelPosition, int opacity, anaPlotAlignVB33 plotLabels, int rangePeriod1, int rangePeriod2, anaSessionCountVB33 selectedSession, bool showCurrentOpen, bool showFibonacci, bool showHiLo, anaPlotAlignVB33 showRangeData, bool showVolatilityBands, bool showVolatilityLabels, int width)
        {
            return _indicator.anaVolatilityBandsV33(Input, bandType, bandWidth, currentSession, includeAfterSession, labelPosition, opacity, plotLabels, rangePeriod1, rangePeriod2, selectedSession, showCurrentOpen, showFibonacci, showHiLo, showRangeData, showVolatilityBands, showVolatilityLabels, width);
        }

        /// <summary>
        /// anaVolatilityBandsV33.
        /// </summary>
        /// <returns></returns>
        public Indicator.anaVolatilityBandsV33 anaVolatilityBandsV33(Data.IDataSeries input, anaBandTypeVB33 bandType, int bandWidth, anaSessionTypeVB33 currentSession, bool includeAfterSession, int labelPosition, int opacity, anaPlotAlignVB33 plotLabels, int rangePeriod1, int rangePeriod2, anaSessionCountVB33 selectedSession, bool showCurrentOpen, bool showFibonacci, bool showHiLo, anaPlotAlignVB33 showRangeData, bool showVolatilityBands, bool showVolatilityLabels, int width)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.anaVolatilityBandsV33(input, bandType, bandWidth, currentSession, includeAfterSession, labelPosition, opacity, plotLabels, rangePeriod1, rangePeriod2, selectedSession, showCurrentOpen, showFibonacci, showHiLo, showRangeData, showVolatilityBands, showVolatilityLabels, width);
        }
    }
}
#endregion
